// 002 u_c_dfm_parser
// 03 feb 2005
(*$r+*)
// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com


unit u_c_dfm_parser;
  interface
    uses Classes, u_c_basic_object
        , u_c_dfm_scanner
        , u_c_dfm_object
        ;

    type c_dfm_parser= Class(c_basic_object)
                         m_trace_parser: Boolean;
                         m_c_dfm_scanner_ref: c_dfm_scanner;
                         m_c_dfm_object: c_dfm_object;
                         m_c_dfm_object_list_ref: c_dfm_object_list;
                         m_pure_parsing: Boolean;

                         constructor create_dfm_parser(p_name: String);

                         procedure display_trace(p_text: String);
                         procedure display_error(p_text: string);
                           function f_symbol_type: t_dfm_symbol_type;
                           function f_symbol_string: String;
                           procedure read_symbol;
                           procedure check(p_dfm_symbol_type: t_dfm_symbol_type);
                           procedure start_parsing;
                         procedure parse_dfm;
                       end; // c_dfm_parser

  implementation
    uses SysUtils, u_c_display, u_strings;

    constructor c_dfm_parser.create_dfm_parser(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_dfm_parser

    procedure c_dfm_parser.display_trace(p_text: String);
      var l_start, l_text: String;
      begin
        if m_trace_parser
          then begin
              l_start:= Copy(p_text, 1, 2);
              l_text:= p_text;
              Delete(l_text, 1, 2);
              with m_c_dfm_scanner_ref do
                display(l_start+ Format('%-7d %-15s %-10s ',
                    [m_buffer_index, m_symbol_string, f_dfm_symbol_name(m_dfm_symbol_type)])
                  + l_text);
            end;
      end; // display_trace

    procedure c_dfm_parser.display_error(p_text: string);
      begin
        display_bug_halt(p_text);
      end; // display_error

    function c_dfm_parser.f_symbol_type: t_dfm_symbol_type;
      begin
        Result:= m_c_dfm_scanner_ref.m_dfm_symbol_type;
      end; // f_symbol_type

    function c_dfm_parser.f_symbol_string: String;
      begin
        Result:= m_c_dfm_scanner_ref.m_symbol_string;
      end; // f_symbol_string

    procedure c_dfm_parser.read_symbol;
      begin
        m_c_dfm_scanner_ref.f_read_symbol;
      end; // read_symbol

    procedure c_dfm_parser.check(p_dfm_symbol_type: t_dfm_symbol_type);
      begin
        if f_symbol_type= p_dfm_symbol_type
          then
          else display_error('expected '+ f_dfm_symbol_name(p_dfm_symbol_type)
            + '  but_found '+ f_dfm_symbol_name(f_symbol_type));
      end; // check

    procedure c_dfm_parser.start_parsing;
      begin
        if not m_c_dfm_scanner_ref.f_initialized
          then display_bug_halt('could_not_initialize_scanner');
      end; // start_parsing

    procedure c_dfm_parser.parse_dfm;

      (*$i i_dfm_parser.pas *)
      (*$i i_pure_dfm_parser.pas *)

      begin // parse_dfm
        start_parsing;
        read_symbol;

        display('parse');
        if m_pure_parsing
          then pure_parse_dfm
          else parse_dfm;
      end; // parse_dfm

  begin // u_c_dfm_parser
  end. // u_c_dfm_parser
